/*
    Fluid test utility.
    Copyright (C) 2017 2018 2021  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <getopt.h>				// getopt_log()
#include "fluid.h"				// movement filters

/***************************************************************************/
/* define vari */

/***************************************************************************/
/* strutture */

/***************************************************************************/
/* costanti */

static const char Help[:>=
	"Usage: %s [OPTIONS] move_data [move_data ...] > testdata.txt\n"
	"creates test a fluid filter.\n\n"
	"\t-h, --help\t\tprints this help.\n"
	"\t-c, --create r,fi,p\tcreates a fluid filter\n"
	"\t-s, --start t,e\tstarts for total,end steps\n"
	"\nReport bugs to bit.trapper@gmail.com\n";

static const char		OptString[:>="+hc:s:";
static const struct option	OptList[:>={
	{"help",	no_argument,		NULL,'h'},
	{"create",	required_argument,	NULL,'c'},
	{"start",	required_argument,	NULL,'s'},
//	{"pippo",	required_argument,	NULL,'M'},
//	{"pappo",	optional_argument,	NULL,'M'},
	{0,0,0,0}				// end of list
};

/***************************************************************************/
/* variabili */

/***************************************************************************/
/* prototipi funzioni */

/***************************************************************************/
/* funzioni */





/***************************************************************************/

int main(int argc,char **argv)
{
  int			Option;			// line command option
  int			OptIdx;			// option index
  unsigned int		Line=0;			// current step
  FluidData_t		Filter;			// filter descriptor
  Fluid_IO_t		Pos;			// posizione iniziale/finale
  Fluid_IO_t		Total;			// total steps
  Fluid_IO_t		In;			// input value
  Fluid_IO_t		Rep;			// # ripetizioni
  Fluid_IO_t		Out;			// filter output value
  float			r,fi;			// modulo e fase dei poli



  // inizializza un filtro di default
  Fluid_Create(&Filter,0.94,0.0,0);
  Fluid_Start(&Filter,10,50);

  // legge le opzioni presenti sulla riga di comando
  while((Option=getopt_long(argc,argv,OptString,OptList,&OptIdx)) >= 0)
  {
    switch(Option)
    {
      case 'h':
        puts(Help);				// help
	return 0;				// termina
      case 'c':
        if(sscanf(optarg,"%f,%f,%hi",&r,&fi,&Pos) != 3)
	{
	  fputs("Unable to decode --create argument\n",stderr);
	  return 1;				// termina con errore
	}
	printf("# Fluid_Create(&Filter,%5.3f,%5.3f,%hi)\n",r,fi,Pos);
	Fluid_Create(&Filter,r,fi,Pos);		// crea il filtro
	break;
      case 's':
        if(sscanf(optarg,"%hi,%hi",&Total,&Pos) != 2)
	{
	  fputs("Unable to decode --start argument\n",stderr);
	  return 2;				// termina con errore
	}
	printf("# Fluid_Start (&Filter,%hi,%hi)\n",Total,Pos);
	Fluid_Start(&Filter,Total,Pos);		// avvia il movimento
	break;
      default:
        fputs("Unknown option, use -h to see help.\n",stderr);
	return 2;				// termina
    }
  }

  // processa tutti gli altri parametri passati
  puts("\n# step\tin\tout\n");
  for(;optind<argc;++optind)
  {
//    printf("processing :%s \n",argv[optind]);
    if(sscanf(argv[optind],"%hi,%hi",&In,&Rep) != 2)
    {
      fputs("Unable to decode move_data argument\n",stderr);
      return 3;					// termina con errore
    }
    while(Rep--)
    {
      Out=Fluid_Move(&Filter,In);		// muove
      printf("%5i\t%hi\t%hi\n",Line++,In,Out);
    }
  }
  return 0;					// termina
}

/* $Source$ */

