/*
    Fluid movement filter test suite.
    Copyright (C) 2017 2018  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "fluid.h"				// movement filters

/***************************************************************************/
/* define vari */

#define TEST_START	-20			// inizio temporale del test
#define TEST_STOP	300			// fine temporale del test

/***************************************************************************/
/* strutture */

/***************************************************************************/
/* costanti */

static const char	Licenza[]=
	"test version 0.02, Copyright (C) 2017 2018  Alessandro Accattini\n";
static const char	Help[]=
	"Usage: test [OPTIONS] pole_r pole_fi [start [stop]]\n"
	"\t-h\tdisplays this help.\n"
	"\t-n sps\tsets the movement length in samples.\n"
	"\t-t %\tenable movement analisys at given percentage.\n"
	"\t-N sps\tsets the number of samples.\n"
	"\nReport bugs to: <bit.trapper@gmail.com>\n";

/***************************************************************************/
/* variabili */

/***************************************************************************/
/* prototipi funzioni */

static void PrintCoeffs(const FluidData_t *Conf);
  /* Stampa a video i coefficienti del filtro. */

/***************************************************************************/
/* funzioni */

static void PrintCoeffs(const FluidData_t *Conf)
{
  /* Stampa a video i coefficienti del filtro. */



  printf("# Out[0]=   0x%08lx\n",Conf->Out[0]);
  printf("# Out[1]=   0x%08lx\n",Conf->Out[1]);
  printf("# Out[2]=   0x%08lx\n",Conf->Out[2]);
  printf("# Coeff[0]= 0x%08lx\n",Conf->Coeff[0]);
  printf("# Coeff[1]= 0x%08lx\n",Conf->Coeff[1]);
  printf("# Coeff[2]= 0x%08lx\n",Conf->Coeff[2]);
/*  
  printf("# Out[0]=   %6.4f\n",Conf->Out[0]);
  printf("# Out[1]=   %6.4f\n",Conf->Out[1]);
  printf("# Out[2]=   %6.4f\n",Conf->Out[2]);
  printf("# Coeff[0]= %6.4f\n",Conf->Coeff[0]);
  printf("# Coeff[1]= %6.4f\n",Conf->Coeff[1]);
  printf("# Coeff[2]= %6.4f\n",Conf->Coeff[2]);
*/
  printf("# MovCount= %hi\n",Conf->MovCount);
  printf("# FinalPos= %hi\n",Conf->FinalPos);
  printf("#\n\n");
  return;				// termina
}





/***************************************************************************/

int main(int argc,char **argv)
{
  int			Opzione;		// per la lettura delle opzioni
  int			i;
  int			N=TEST_STOP;		// campioni analizzati
  Fluid_IO_t		Reset=0;		// valore iniziale del filtro
  Fluid_IO_t		In=100;			// ingresso filtro
  Fluid_IO_t		Out;			// uscita del filtro
  Fluid_IO_t		n=50;			// durata del movimento
  FluidData_t		Fluid;			// filter descriptor
  float			r,fi;			// modulo e fase dei poli
  float			TestThr=0.0;		// soglia di test
 



  // legge le opzioni presenti sulla riga di comando
  while((Opzione=getopt(argc,argv,"hN:n:t:"))!=-1)
  {
    switch(Opzione)
    {
      case 'h':
        puts(Help);				// stampa l'help
	return 0;				// termina
	break;
      case 'n':
        sscanf(optarg,"%hi",&n);		// imposta la durata movimento
        break;
      case 'N':
        sscanf(optarg,"%i",&N);			// imposta la durata del test
        break;
      case 't':
        sscanf(optarg,"%f",&TestThr);		// la soglia
        break;
      default:
        puts("Opzione sconosciuta !!!");
	exit(0);				// termina
	break;
    }
  }

/*
  // processa tutti gli altri parametri passati
  for(;optind<argc;++optind)
  {
    printf("processing :%s \n",argv[optind]);
  }
*/

  // verifica di avere almeno altri due parametri
  if((argc - optind) < 2)
  {
    printf("Mancano i parametri!!\n");
    return 0;
  }
  sscanf(argv[optind++],"%f",&r);
  sscanf(argv[optind++],"%f",&fi);
  if(TestThr == 0.0)				// se test non attivo
  {
    // visualizza la licenza del programma
    printf("# %s\n\n",Licenza);

    printf("# r,fi = %6.3f,%6.3f\n#\n",r,fi);
  }
  if(optind < argc)				// se c'e' anche START
    sscanf(argv[optind++],"%hi",&Reset);	// punto di partenza
  if(optind < argc)				// se c'e' anche STOP
    sscanf(argv[optind++],"%hi",&In);		// punto di arrivo
  Out=Reset;					// uscita iniziale (t<0)



  Fluid_Create(&Fluid,r,fi,Reset);		// init del filtro
  Fluid_Start(&Fluid,n,In);			// avvia il movimento
  if(TestThr == 0.0)				// se test non attivo
  {
    PrintCoeffs(&Fluid);			// stampa i coefficienti

    // pre-test, inserisce dei valori costanti prima di 0
    for(i=TEST_START;i <= 0;i++)
      printf("%4i\t%6hi\t%6hi\n",i,In,Out);
  }
	   
  // test del filtro
  for(i=1;i <= N;i++)
  {
    Out=Fluid_Move(&Fluid,In);			// uscita del filtro
    if(TestThr == 0.0)				// se test non attivo
    {
      printf("%4i\t%6hi\t%6hi\n",i,In,Out);
    }
    else
    {
      if((float)Out >= (TestThr * In))
      {
        printf("%5.3f\t%6.3f\t%i\n",r,fi,i);
	return 0;				// termina
      }
    }
  }
  return 0;					// termina
}

/* $Source$ */

