/*
    Sequencer implementation.
    Copyright (C) 2021  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#include <stddef.h>				// NULL

#include "sequencer.h"				// functions prototypes

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

void Sequencer_Run(Sequencer_t *Seq,void *Data)
{
  /* Runs sequencer's sequence single cut step. */
  
  
  
  SeqCut_t	ActiveCut;			// current active cut
  
  
  
  ActiveCut=Seq->Sequence[Seq->Active];		// active call
  if(!ActiveCut(Data,Seq->Frame++))
  {
    // current cut is ended, we have to switch to the next one
    Seq->Frame=0;				// prima chiamata
    Seq->Active++;				// cut successivo
    if(Seq->Sequence[Seq->Active] == NULL)	// se e' la fine sequenza
      Seq->Active=0;				// riparte dal primo cut
  }
  return;					// termina
}



void Sequencer_Rewind(Sequencer_t *Seq,unsigned int Cuts)
{
  /* Restarts sequence execution from a previeus cut. */

  
  
  if(!Cuts)					// se nullo
    return;					// ignora
  if(Cuts >= Seq->Active)			// se sfora all'indietro
    Seq->Active=0;				// inizio sequenza
  else
    Seq->Active-=Cuts;				// riavvolge la sequenza
  Seq->Frame=0;					// prima chiamata
  return;					// termina
}



void Sequencer_FastForward(Sequencer_t *Seq,unsigned int Cuts)
{
  /* Moves sequence execution to a furter cut. */

  
  
  if(!Cuts)					// se nullo
    return;					// ignora
  do
  {
    Seq->Active++;				// salta al prossimo
    if(Seq->Sequence[Seq->Active] == NULL)	// se e' il marker fine seq.
    {
      Seq->Active=0;				// riparte da inizio sequenza
      break;					// fine della storia
    }
  }while(--Cuts);
  Seq->Frame=0;					// prima chiamata
  return;					// termina
}

/* $Source$ */

